package com.example.vp3300_sdk_tutorial;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Set;

import android.app.Activity;
import android.app.Dialog;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;

import com.idtechproducts.device.*;
import com.idtechproducts.device.ReaderInfo.CAPTURE_ENCODE_TYPE;
import com.idtechproducts.device.ReaderInfo.CAPTURE_ENCRYPT_TYPE;
import com.idtechproducts.device.ReaderInfo.DEVICE_TYPE;
import com.idtechproducts.device.ReaderInfo.EVENT_MSR_Types;

public class MainActivity extends Activity implements  OnReceiverListener {

	// declaring the instance of the UniPayReader;
	private IDT_VP3300 myUniPayReader = null;
	private TextView connectStatusTextView;  
	private TextView textLog;
	private TextView lcdLog;
	private Button btnGetFirmware;
	private Button btnStartSwipe;
	private Button btnCancelSwipe;
	private Button btnStartEMV;
	private Button btnCancelEMV;
	private Handler handler = new Handler();
    private boolean isReaderConnected = false;
    private String info = "";
	private String detail = "";
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		handler = new Handler();		
		btnStartEMV = (Button)findViewById(R.id.btn_StartEMV);
		btnCancelEMV = (Button)findViewById(R.id.btn_CancelEMV);
		btnGetFirmware = (Button)findViewById(R.id.btn_getFirmware);
		btnStartSwipe = (Button)findViewById(R.id.btn_startSwipe);
		btnCancelSwipe = (Button)findViewById(R.id.btn_cancelSwipe);
	    textLog = (TextView)findViewById(R.id.textLog);
	    lcdLog = (TextView)findViewById(R.id.lcdLog);
	    connectStatusTextView = (TextView)findViewById(R.id.status_text);
		if(myUniPayReader!=null){
			myUniPayReader.unregisterListen();
			myUniPayReader.release();
			myUniPayReader = null;
		}

		myUniPayReader = new IDT_VP3300(this,this);

		//myUniPayReader.device_setDeviceType(DEVICE_TYPE.DEVICE_VP3300_AJ);

		myUniPayReader.registerListen();
		loadXMLfile();
		

		btnGetFirmware.setOnClickListener(new Button.OnClickListener() {
	        public void onClick(View v) {
	        	info = "Getting Firmware\n";
	        	detail = "";
	        	handler.post(doUpdateStatus);
	        	StringBuilder sb = new StringBuilder();
	            int ret = myUniPayReader.device_getFirmwareVersion(sb);
	            if (ret == ErrorCode.SUCCESS) {
					info += "Firmware Version: " + sb.toString();
					detail = "";
					handler.post(doUpdateStatus);
				}
				else {
					info += "GetFirmwareVersion: Failed\n";
					info += "Status: "+ myUniPayReader.device_getResponseCodeString(ret)+"";
					detail = "";
					handler.post(doUpdateStatus);							
					}
	        }  
	    });
		
		btnStartEMV.setOnClickListener(new Button.OnClickListener() {
	        public void onClick(View v) {
	        detail = "";
	        info = "Starting EMV Transaction\n";
        	handler.post(doUpdateStatus);

        	IDT_VP3300.emv_allowFallback(true);

				myUniPayReader.emv_startTransaction(1.00, 0.00, 0, 30, null, false);
	        }  
	    });
		
		btnCancelEMV.setOnClickListener(new Button.OnClickListener() {
	        public void onClick(View v) {
	        	detail = "";
		        info = "Canceling EMV Transaction\n";
	        	handler.post(doUpdateStatus);
	        	ResDataStruct resData = new ResDataStruct();
				myUniPayReader.emv_cancelTransaction(resData);
	        }  
	    });
		
		btnStartSwipe.setOnClickListener(new Button.OnClickListener() {
	        public void onClick(View v) {
	        	detail = "";
		        info = "Starting Swipe/Tap Transaction\n";
	        	handler.post(doUpdateStatus);
				myUniPayReader.msr_startMSRSwipe();
	        }  
	    });
		
		btnCancelSwipe.setOnClickListener(new Button.OnClickListener() {
	        public void onClick(View v) {
	        	detail = "";
		        info = "Cancelling Swipe/Tap Transaction\n";
	        	handler.post(doUpdateStatus);
				myUniPayReader.msr_cancelMSRSwipe();
	        }  
	    });
		
	}

	@Override
	public void ICCNotifyInfo(byte[] arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void LoadXMLConfigFailureInfo(int arg0, String arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void autoConfigCompleted(StructConfigParameters arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void autoConfigProgress(int arg0) {
		// TODO Auto-generated method stub
		
	}
	
	private Runnable doUpdateLabel = new Runnable() {
	    public void run() {
	        if(!isReaderConnected) {
	            connectStatusTextView.setText("VP3300 AJ DISCONNECTED");
	        }
	        else {
	            connectStatusTextView.setText("VP3300 AJ CONNECTED");
	        }
	    }
	};
	@Override
	public void deviceConnected() {
		isReaderConnected = true;
		handler.post(doUpdateLabel);		
	}

	@Override
	public void deviceDisconnected() {
		isReaderConnected = false;
	    handler.post(doUpdateLabel);		
	}
	
	private void printTags(IDTEMVData emvData) {
	
	}
	
	@Override
	public void emvTransactionData(IDTEMVData emvData) {
		
		detail += this.emvErrorCodes(emvData.result);
		detail += "\r\n";
		if (emvData.result == IDTEMVData.START_TRANS_SUCCESS)
			detail += "Start transaction response:\r\n";
		else if (emvData.result == IDTEMVData.GO_ONLINE)
			detail += "\r\nAuthentication response:\r\n";
		else 
			detail += "\r\nComplete Transaction response:\r\n";
		if (emvData.unencryptedTags != null && !emvData.unencryptedTags.isEmpty()) {
			detail += "Unencrypted Tags:\r\n";
			Set<String> keys = emvData.unencryptedTags.keySet();
			for(String key: keys) {
				detail += key + ": ";
				byte[] data = emvData.unencryptedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		if (emvData.maskedTags!= null && !emvData.maskedTags.isEmpty()) {
			detail += "Masked Tags:\r\n";
			Set<String> keys = emvData.maskedTags.keySet();
			for(String key: keys) {
				detail += key + ": ";
				byte[] data = emvData.maskedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		if (emvData.encryptedTags != null && !emvData.encryptedTags.isEmpty()) {
			detail += "Encrypted Tags:\r\n";
			Set<String> keys = emvData.encryptedTags.keySet();
			for(String key: keys) {
				detail += key + ": ";
				byte[] data = emvData.encryptedTags.get(key);
				detail += Common.getHexStringFromBytes(data) + "\r\n";
			}
		}
		handler.post(doUpdateStatus);
		 if (emvData.result == IDTEMVData.GO_ONLINE) {
			//Auto Complete
			 byte[] response = new byte[]{0x30,0x30};
			 myUniPayReader.emv_completeTransaction(false, response, null, null,null);
		 }
		 else if (emvData.result == IDTEMVData.START_TRANS_SUCCESS) {
				//Auto Authenticate
			 myUniPayReader.emv_authenticateTransaction(null);
			}
	
	}

	public void lcdDisplay(int mode, String[] lines, int timeout) {
		
				if (mode == 0x01) //Menu Display
				{
					//automatically select 1st application
					myUniPayReader.emv_lcdControlResponse((byte)mode, (byte)0x01);
				}
				else if (mode == 0x08) //Language Menu Display
				{
					//automatically select first language
					myUniPayReader.emv_lcdControlResponse((byte)mode, (byte)0x01);
				}
				else{
					ResDataStruct toData = new ResDataStruct();
					info = lines[0];
					handler.post(doUpdateStatus);
				}			
	}


	@Override
	public void msgAudioVolumeAjustFailed() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgRKICompleted(String arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgToConnectDevice() {
		// TODO Auto-generated method stub
		
	}

	private Runnable doUpdateStatus = new Runnable() {
		public void run()
		{		
			lcdLog.setText(info);
			textLog.setText(detail);
		}
	};
	@Override
	public void swipeMSRData(IDTMSRData card) {
		if (card.cardData[0] != (byte)0x01 && card.track1Length == 0 && card.track2Length == 0 && card.track3Length == 0)
			info = "Swipe/Tap data didn't read correctly";
		else
			info = "Swipe/Tap Read Successfully";
		detail = Common.parse_MSRData(myUniPayReader.device_getDeviceType(), card);
		handler.post(doUpdateStatus);
		
	}

	@Override
	public void timeout(int arg0) {
		// TODO Auto-generated method stub
		
	}
	
	private String getXMLFileFromRaw(String fileName ,int res) {
		//the target filename in the application path
		String fileNameWithPath = null;
		fileNameWithPath = fileName;
		String newFilename = fileName;
	
		try {
			InputStream in = getResources().openRawResource(res);
			int length = in.available();
			byte [] buffer = new byte[length];
			in.read(buffer);    	   
			in.close();
			deleteFile(fileNameWithPath);
			FileOutputStream fout = openFileOutput(fileNameWithPath, MODE_PRIVATE);
			fout.write(buffer);
			fout.close();
    	   
			// to refer to the application path
			File fileDir = this.getFilesDir();
			fileNameWithPath = fileDir.getParent() + java.io.File.separator + fileDir.getName();
			fileNameWithPath += java.io.File.separator+newFilename;
	   	   
		} catch(Exception e) {
			e.printStackTrace();
			fileNameWithPath = null;
		}
		return fileNameWithPath;
	}
	
	private String getConfigurationFileFromRaw( ) {
		return getXMLFileFromRaw("idt_unimagcfg_default.xml",R.raw.idt_unimagcfg_default);
	}
	private boolean isFileExist(String path) {
    	if(path==null)
    		return false;
	    File file = new File(path);
	    if (!file.exists()) {
	      return false ;
	    }
	    return true;
    }
	private void loadXMLfile() {
		
	    //load the XML configuratin file
	    String fileNameWithPath = getConfigurationFileFromRaw();
	    if(!isFileExist(fileNameWithPath)) { 
	    	fileNameWithPath = null; 
	    }        	
	    /////////////////////////////////////////////////////////////////////////////////
		// Network operation is prohibited in the UI Thread if target API is 11 or above.
		// If target API is 11 or above, please use AsyncTask to avoid errors.
	    myUniPayReader.config_setXMLFileNameWithPath(fileNameWithPath);
	    Log.d("Demo Info >>>>>","loadingConfigurationXMLFile begin.");
	    myUniPayReader.config_loadingConfigurationXMLFile(true);
	    /////////////////////////////////////////////////////////////////////////////////
	}

	@Override
	public void dataInOutMonitor(byte[] arg0, boolean arg1) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void msgBatteryLow() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void lcdDisplay(int arg0, String[] arg1, int arg2, byte[] arg3,
			byte arg4) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void ctlsEvent(byte arg0, byte arg1, byte arg2) {
		// TODO Auto-generated method stub
		
	}
	
	
	private String emvErrorCodes(int val) {
		if (val ==IDTEMVData.APPROVED_OFFLINE) return "APPROVED_OFFLINE";
		if (val ==IDTEMVData.DECLINED_OFFLINE) return "DECLINED_OFFLINE";
		if (val ==IDTEMVData.APPROVED) return "APPROVED";
		if (val ==IDTEMVData.DECLINED) return "DECLINED";
	    if (val ==IDTEMVData.GO_ONLINE) return "GO_ONLINE";
	    if (val ==IDTEMVData.CALL_YOUR_BANK) return "CALL_YOUR_BANK";
	    if (val ==IDTEMVData.NOT_ACCEPTED) return "NOT_ACCEPTED";
	    if (val ==IDTEMVData.USE_MAGSTRIPE) return "USE_MAGSTRIPE";
	    if (val ==IDTEMVData.TIME_OUT) return "TIME_OUT";
	    if (val ==IDTEMVData.GO_ONLINE_CTLS) return "GO_ONLINE_CTLS";
	    if (val ==IDTEMVData.START_TRANS_SUCCESS) return "START_TRANS_SUCCESS";
	    if (val ==IDTEMVData.MSR_SUCCESS) return "MSR_SUCCESS";
	    if (val ==IDTEMVData.TRANSACTION_CANCELED) return "TRANSACTION_CANCELED";
        if (val ==IDTEMVData.CTLS_TWO_CARDS) return "CTLS_TWO_CARDS";
        if (val ==IDTEMVData.CTLS_TERMINATE) return "CTLS_TERMINATE";
        if (val ==IDTEMVData.CTLS_TERMINATE_TRY_ANOTHER) return "CTLS_TERMINATE_TRY_ANOTHER";
        if (val ==IDTEMVData.MSR_SWIPE_CAPTURED) return "MSR_SWIPE_CAPTURED";
        if (val ==IDTEMVData.REQUEST_ONLINE_PIN) return "REQUEST_ONLINE_PIN";
        if (val ==IDTEMVData.REQUEST_SIGNATURE) return "REQUEST_SIGNATURE";
        if (val ==IDTEMVData.FALLBACK_TO_CONTACT) return "FALLBACK_TO_CONTACT";
        if (val ==IDTEMVData.FALLBACK_TO_OTHER) return "FALLBACK_TO_OTHER";
        if (val ==IDTEMVData.REVERSAL_REQUIRED) return "REVERSAL_REQUIRED";
        if (val ==IDTEMVData.ADVISE_REQUIRED) return "ADVISE_REQUIRED";
        if (val ==IDTEMVData.NO_ADVISE_REVERSAL_REQUIRED) return "NO_ADVISE_REVERSAL_REQUIRED";
        if (val ==IDTEMVData.UNABLE_TO_REACH_HOST) return "UNABLE_TO_REACH_HOST";
	    if (val ==IDTEMVData.FILE_ARG_INVALID) return "FILE_ARG_INVALID";
	    if (val ==IDTEMVData.FILE_OPEN_FAILED) return "FILE_OPEN_FAILED";
	    if (val ==IDTEMVData.FILE_OPERATION_FAILED) return "FILE_OPERATION_FAILED";
	    if (val ==IDTEMVData.MEMORY_NOT_ENOUGH) return "MEMORY_NOT_ENOUGH";
        if (val ==IDTEMVData.SMARTCARD_OK) return "SMARTCARD_OK";
	    if (val ==IDTEMVData.SMARTCARD_FAIL) return "SMARTCARD_FAIL";
	    if (val ==IDTEMVData.SMARTCARD_INIT_FAILED) return "SMARTCARD_INIT_FAILED";
	    if (val ==IDTEMVData.FALLBACK_SITUATION) return "FALLBACK_SITUATION";
	    if (val ==IDTEMVData.SMARTCARD_ABSENT) return "SMARTCARD_ABSENT";
	    if (val ==IDTEMVData.SMARTCARD_TIMEOUT) return "SMARTCARD_TIMEOUT";
	    if (val ==IDTEMVData.MSR_CARD_ERROR) return "MSR_CARD_ERROR";
	    if (val ==IDTEMVData.MSR_CARD_ERROR_FALLBACK) return "MSR_CARD_ERROR_FALLBACK";
	    if (val ==IDTEMVData.TIMEOUT_FOR_WAITING_ICC_INSERT_OR_MSR_SWIPE_FALLBACK) return "TIMEOUT_FOR_WAITING_ICC_INSERT_OR_MSR_SWIPE_FALLBACK";
	    if (val ==IDTEMVData.PARSING_TAGS_FAILED) return "PARSING_TAGS_FAILED";
	    if (val ==IDTEMVData.CARD_DATA_ELEMENT_DUPLICATE) return "CARD_DATA_ELEMENT_DUPLICATE";
	    if (val ==IDTEMVData.DATA_FORMAT_INCORRECT) return "DATA_FORMAT_INCORRECT";
	    if (val ==IDTEMVData.APP_NO_TERM) return "APP_NO_TERM";
	    if (val ==IDTEMVData.APP_NO_MATCHING) return "APP_NO_MATCHING";
	    if (val ==IDTEMVData.AMANDATORY_OBJECT_MISSING) return "AMANDATORY_OBJECT_MISSING";
	    if (val ==IDTEMVData.APP_SELECTION_RETRY) return "APP_SELECTION_RETRY";
	    if (val ==IDTEMVData.AMOUNT_ERROR_GET) return "AMOUNT_ERROR_GET";
	    if (val ==IDTEMVData.CARD_REJECTED) return "CARD_REJECTED";
	    if (val ==IDTEMVData.AIP_NOT_RECEIVED) return "AIP_NOT_RECEIVED";
	    if (val ==IDTEMVData.AFL_NOT_RECEIVEDE) return "AFL_NOT_RECEIVEDE";
	    if (val ==IDTEMVData.AFL_LEN_OUT_OF_RANGE) return "AFL_LEN_OUT_OF_RANGE";
	    if (val ==IDTEMVData.SFI_OUT_OF_RANGE) return "SFI_OUT_OF_RANGE";
	    if (val ==IDTEMVData.AFL_INCORRECT) return "AFL_INCORRECT";
	    if (val ==IDTEMVData.EXP_DATE_INCORRECT) return "EXP_DATE_INCORRECT";
	    if (val ==IDTEMVData.EFF_DATE_INCORRECT) return "EFF_DATE_INCORRECT";
	    if (val ==IDTEMVData.ISS_COD_TBL_OUT_OF_RANGE) return "ISS_COD_TBL_OUT_OF_RANGE";
	    if (val ==IDTEMVData.CRYPTOGRAM_TYPE_INCORRECT) return "CRYPTOGRAM_TYPE_INCORRECT";
	    if (val ==IDTEMVData.PSE_BY_CARD_NOT_SUPPORTED) return "PSE_BY_CARD_NOT_SUPPORTED";
	    if (val ==IDTEMVData.USER_LANGUAGE_SELECTED) return "USER_LANGUAGE_SELECTED";
	    if (val ==IDTEMVData.SERVICE_NOT_ALLOWED) return "SERVICE_NOT_ALLOWED";
	    if (val ==IDTEMVData.NO_TAG_FOUND) return "NO_TAG_FOUND";
	    if (val ==IDTEMVData.CARD_BLOCKED) return "CARD_BLOCKED";
	    if (val ==IDTEMVData.LEN_INCORRECT) return "LEN_INCORRECT";
	    if (val ==IDTEMVData.CARD_COM_ERROR) return "CARD_COM_ERROR";
	    if (val ==IDTEMVData.TSC_NOT_INCREASED) return "TSC_NOT_INCREASED";
	    if (val ==IDTEMVData.HASH_INCORRECT) return "HASH_INCORRECT";
	    if (val ==IDTEMVData.ARC_NOT_PRESENCED) return "ARC_NOT_PRESENCED";
	    if (val ==IDTEMVData.ARC_INVALID) return "ARC_INVALID";
	    if (val ==IDTEMVData.COMM_NO_ONLINE) return "COMM_NO_ONLINE";
	    if (val ==IDTEMVData.TRAN_TYPE_INCORRECT) return "TRAN_TYPE_INCORRECT";
	    if (val ==IDTEMVData.APP_NO_SUPPORT) return "APP_NO_SUPPORT";
	    if (val ==IDTEMVData.APP_NOT_SELECT) return "APP_NOT_SELECT";
	    if (val ==IDTEMVData.LANG_NOT_SELECT) return "LANG_NOT_SELECT";
	    if (val ==IDTEMVData.TERM_DATA_NOT_PRESENCED) return "TERM_DATA_NOT_PRESENCED";
	    if (val ==IDTEMVData.CVM_TYPE_UNKNOWN) return "CVM_TYPE_UNKNOWN";
	    if (val ==IDTEMVData.CVM_AIP_NOT_SUPPORTED) return "CVM_AIP_NOT_SUPPORTED";
	    if (val ==IDTEMVData.CVM_TAG_8E_MISSING) return "CVM_TAG_8E_MISSING";
	    if (val ==IDTEMVData.CVM_TAG_8E_FORMAT_ERROR) return "CVM_TAG_8E_FORMAT_ERROR";
	    if (val ==IDTEMVData.CVM_CODE_IS_NOT_SUPPORTED) return "CVM_CODE_IS_NOT_SUPPORTED";
	    if (val ==IDTEMVData.CVM_COND_CODE_IS_NOT_SUPPORTED) return "CVM_COND_CODE_IS_NOT_SUPPORTED";
	    if (val ==IDTEMVData.CVM_NO_MORE) return "CVM_NO_MORE";
	    if (val ==IDTEMVData.PIN_BYPASSED_BEFORE) return "PIN_BYPASSED_BEFORE";
        if (val ==IDTEMVData.UNKONWN) return "UNKONWN";
		return "";
	}

	
	
}
